#!/usr/bin/perl -w
###############################################################
# Simple Vote Counter                         Version: v1.01  #
###############################################################
# Written By:   Mark R Hives                                  #
# Filename:     m_vote.pl                                     #
# Release Date: 08-Apr-2004                                   #
#               05-Feb-2003  [v1.00]                          #
#                                                             #
# Description:                                                #
#  A simple voting system that is relatively easy to update   #
#  once installed and prevents casual users (non-hackers)     #
#  from voting more than once.                                #
#                                                             #
# How to use this script:                                     #
#  To install create a directory on your web server to        #
#  contain the support files (usually called 'm_vote_data').  #
#  Next create an empty file called 'lock-file' within it     #
#  and change the '$sd' variable, in this script, to contain  #
#  the path to the new directory. Finally upload the          #
#  modified script to your cgi-bin.                           #
#                                                             #
#  Please note for improved security you may wish to change   #
#  the '$reset_code' variable below.                          #
#                                                             #
#  To make resetting the voting system easier I have          #
#  included a file called 'vote_reset.htm'. Simply fill-in    #
#  this form and it will generate a list of examples and a    #
#  reset button.                                              #
###############################################################

my (%fd, %cd, %votes, %names, $sd, $reset_code, @tl, $x,
    $html_title, $html_header, $html_footer, $cookie_exp);

###############################################################
# CONFIGURATION SECTION                                       #
###############################################################

#Path to Support Directory
$sd          = "./m_vote_data";

#Vote system Reset Code
$reset_code  = ".qWeRtYQwErTy";

#Title, Header and Footer
$html_title  = "Vote Counter";
$html_header = "";
$html_footer = "";

#Number of Seconds Between Votes
$cookie_exp  = 86400;

###############################################################


#Get and parse User data
%cd = ();
read_cookies();
%fd = ();
read_formdata();


#Hashes and Locks - OPEN
open(LOCK_FILE, "+< $sd/lock-file") or die_msg("OPEN LOCKFILE - Failed");
flock(LOCK_FILE, 2) or die_msg("FLOCK 2 - Failed");
dbmopen(%votes, "$sd/votes", 0666) or die_msg("DBMOPEN votes - Failed");
dbmopen(%names, "$sd/names", 0666) or die_msg("DBMOPEN names - Failed");


##
if ($fd{'vote'} eq $reset_code) {
   if ($fd{'names'}) {
      @tl = split(/\|/, $fd{'names'});
      die_msg("Invalid names length - Reset Failed", @tl) if (scalar(@tl) % 2);

      %names = (@tl);
      %votes = ();

      foreach $x (keys %names) {
         $votes{$x} = 1;
      }

      output_header();
      #foreach $x (@tl) {
      #   print "<br>", $x;
      #}
      show_results();
      output_footer();

   } else {
      die_msg("No names - Reset Failed");
   }


} elsif ($fd{'vote'} eq '.show') {
   output_header();
   show_results();
   output_footer();

} elsif ($names{$fd{'vote'}}) {
   output_cookie();
   output_header();
   if ($cd{'voted_for'}) {
      print "<p>Sorry, You can only vote once.</p>";
   } else {
      $votes{$fd{'vote'}}++;
   }
   show_results();
   output_footer();

} else {
   die_msg("vote not valid !!!", $fd{'vote'});
}


#Hashes and Locks - CLOSE
dbmclose(%votes) or die_msg("DBMCLOSE votes - Failed");
dbmclose(%names) or die_msg("DBMCLOSE names - Failed");
flock(LOCK_FILE, 8) or die_msg("FLOCK 8 - Failed");
close LOCK_FILE;

############################################################################

sub format_time {
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)
   = (gmtime($_[0] || time()));
  return sprintf("%s, %d-%s-%d %02d:%02d:%02d GMT",
         (qw(Sun Mon Tue Wed Thu Fri Sat))[$wday], $mday,
         (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec))[$mon],
         ($year + 1900) ,
         $hour,$min,$sec);
}

sub show_results {
   my($x, @keylist, $total);
   print "<table border=1 width=600>\n";
   @keylist = keys %names;
   $total = 0;
   foreach $x (@keylist) {
      $total = $total + $votes{$x};
   }
   foreach $x (sort @keylist) {
      print "<tr><td>", $x, "</td>\n",
            "    <td>", $names{$x}, "</td>\n",
            "    <td align=right>";
      printf("%.1f %%</td>\n", ($votes{$x} * 100 / $total));
   }
   print "</table>\n";
}



sub die_msg    {
   output_header();
   print @_;
   #print %ENV;
   output_footer();
   exit;
}

sub read_formdata {
   my (@pairs, $pair, $name, $value, $buffer);
   if ($ENV{'REQUEST_METHOD'} eq 'POST') {
      read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
      @pairs = split /&/, $buffer;
   } else {
      @pairs = split /&/, $ENV{'QUERY_STRING'};
   }

   foreach $pair (@pairs) {
      ($name, $value) = split /=/, $pair;
      $name =~ tr/+/ /;
      $name =~ s/%([\da-f]{2})/chr(hex($1))/egi;
      $value =~ tr/+/ /;
      $value =~ s/%([\da-f]{2})/chr(hex($1))/egi;

      #$value =~ tr/\|\\\/;/____/;
      $fd{$name} = $value;
   }
}

sub read_cookies   {
   my (@pairs, $pair, $name, $value);
   @pairs = split /; /, $ENV{'HTTP_COOKIE'};
   foreach $pair (@pairs) {
      ($name, $value) = split /=/, $pair;
      $value =~ tr/+/ /;
      $value =~ s/%([\da-f]{2})/chr(hex($1))/egi;

      $cd{$name} = $value;
   }
}

sub output_cookie {
   print "Set-Cookie: voted_for=", $fd{'vote'},
                   "; expires=", format_time(time() + $cookie_exp), "\n";
}

sub output_header {
   print "content-type: text/html\n\n";
   print "<html>\n<head>\n<title>", $html_title, "</title>\n</head>\n",
         "<body>\n";
   print "<table width=100% border=0><tr><td>\n",
         "<font size=5 color=#333333>", $html_title, "</font>",
         "<br>", $html_header,
         "</td></tr></table>\n";
}

sub output_footer {
   print "<table width=100% border=0 cellpadding=0 cellspacing=0>\n",
         "<tr><td colspan=2>", $html_footer, "</td></tr>\n",
         "<tr><td align=left width=50%>\n",
         "<font size=1 color=#999999>Simple Vote Counter v1.00</font>\n",
         "</td><td align=right width=50%>",
         "<font size=1 color=#999999>&copy; 2003 Mark R Hives</font>\n",
         "</td></tr></table>\n";

   #print "\n<code><br>\n", $ENV{'HTTP_COOKIE'}, "\n</code>\n"; ######################
   print "</body>\n</html>";
}


